from typing import Any
from django.db.models.query import QuerySet
from django.db.models import Q
from django.forms.models import BaseModelForm
from django.http import HttpResponse
from django.shortcuts import render
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse
from django.views.generic import (
    ListView, 
    DetailView, 
    UpdateView, 
    DeleteView, 
    CreateView
)
from dj_tickets.core.models import Ticket
from dj_tickets.core.templatetags.dj_tickets_core_tags import dj_tickets_settings_page_sizes
from dj_tickets.core.utils import check_user_has_support_member
from .forms import (
    AdminTicketUpdateForm,
    ClientTicketUpdateForm,
    SupportMemberTicketUpdateForm, 
    TicketCreateForm,
    ClientAdminTicketUpdateForm,
    ClientSupportMemberTicketUpdateForm
)

# Create your views here.

class TicketsListView(LoginRequiredMixin,ListView):
    queryset = Ticket.objects.all()
    paginate_by= min(*dj_tickets_settings_page_sizes())

    def get_queryset(self) -> QuerySet[Any]:
        if check_user_has_support_member(self.request.user):
            if check_user_has_support_member(self.request.user,True):
                # admin user
                return super().get_queryset().filter(
                    Q(support_team=self.request.user.support_team_member.support_team) |
                    Q(support_team__isnull=True)
                )
            # support member user
            return super().get_queryset().filter(support_team_member=self.request.user.support_team_member)
        #client user
        return super().get_queryset().filter(creator=self.request.user)
    
    def get_template_names(self) -> list[str]:
        if check_user_has_support_member(self.request.user):
            if check_user_has_support_member(self.request.user,True):
                # admin user
                return ["dj_tickets/mtv/admin_tickets_page.html"]
            # support member user
            return ["dj_tickets/mtv/support_member_tickets_page.html"]
        #client user
        return ["dj_tickets/mtv/client_tickets_page.html"]
    
    
    def get_paginate_by(self, queryset) -> int | None:
        # Allow users to change the page size dynamically
        return self.request.GET.get('page_size', self.paginate_by)  # Default to 5 if not specified


class TicketUpdateView(LoginRequiredMixin,UpdateView):

    queryset = Ticket.objects.all()

    def get_form_class(self) -> type[BaseModelForm]:
        
        
        if check_user_has_support_member(self.request.user):
            if check_user_has_support_member(self.request.user,True):
                if self.get_object().creator == self.request.user:
                    #client user is the owner
                    return ClientAdminTicketUpdateForm
                # admin user
                return AdminTicketUpdateForm
            
            if self.get_object().creator == self.request.user:
                #client user is the owner
                return ClientSupportMemberTicketUpdateForm
            # support member user
            return SupportMemberTicketUpdateForm
        # client user
        return ClientTicketUpdateForm
    
    def get_template_names(self) -> list[str]:
        return ["dj_tickets/mtv/admin_ticket_update_page.html"]
        if check_user_has_support_member(self.request.user):
            if check_user_has_support_member(self.request.user,True):
                # admin user
                return ["dj_tickets/mtv/admin_ticket_update_page.html"]
            # support member user
            return ["dj_tickets/mtv/support_member_ticket_update_page.html"]
        #client user
        return ["dj_tickets/mtv/client_ticket_update_page.html"]

    def get_success_url(self) -> str:
        return reverse('dj_tickets:tickets-list')


class TicketCreateView(LoginRequiredMixin,CreateView):
    template_name = "dj_tickets/mtv/ticket_create_page.html"
    queryset = Ticket.objects.all()
    form_class = TicketCreateForm
    
    def form_valid(self, form: BaseModelForm) -> HttpResponse:
        form.instance.creator = self.request.user
        return super().form_valid(form)

    def get_success_url(self) -> str:
        return reverse('dj_tickets:tickets-list')


class TicketDetailView(LoginRequiredMixin,DetailView):
    template_name = "dj_tickets/mtv/ticket_chat_page.html"
    queryset = Ticket.objects.all()



