from rest_framework import serializers
from django.apps import apps
from django.conf import settings
from dj_tickets.core.models import (
    Ticket, 
    AbstractTicket , 
    SupportTeam , 
    SupportTeamMember,
    TicketMessage,
)
from dj_tickets.core.serializers import UserDetailSerializer
from django.utils.module_loading import import_string
from dj_tickets.core.utils import get_settings


class BaseClientTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',

            'content',
            'observers',
            'support_team',

            'status',
            'support_team_member',
            'created',
            'modified',
        ]
        extra_kwargs = {
            'support_team_member': { 'read_only':True },
            'status': { 'read_only':True },
        }

    def to_representation(self, instance:AbstractTicket):
        data = super().to_representation(instance)

        data['observers'] = UserDetailSerializer(instance.observers,many=True).data

        return data
    
    def create(self, validated_data):
        validated_data['creator'] = self.context['request'].user
        return super().create(validated_data)

    def update(self, instance: AbstractTicket, validated_data):
        instance.observers.add(*validated_data.pop('observers',[]))
        return super().update(instance, validated_data)


ClientTicketSerializer : type[BaseClientTicketSerializer] = import_string(get_settings().get('ClientTicketSerializer','dj_tickets.rest.serializers.BaseClientTicketSerializer'))




class BaseSupportTeamMemberTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',

            'content',
            'observers',
            'support_team',
            'creator',
            'status',
            'support_team_member',
            'created',
            'modified',
        ]
        extra_kwargs = {
            'content': { 'read_only':True },
            'support_team': { 'read_only':True },
            'creator': { 'read_only':True },
            'support_team_member': { 'read_only':True },
        }

    def to_representation(self, instance:AbstractTicket):
        data = super().to_representation(instance)

        data['observers'] = UserDetailSerializer(instance.observers,many=True).data
        data['creator'] = UserDetailSerializer(instance.creator).data

        return data

    def update(self, instance: AbstractTicket, validated_data):
        if validated_data.get('status',None) == Ticket.Status.PENDING:
            validated_data.pop('status',None)

        instance.observers.add(*validated_data.pop('observers',[]))

        return super().update(instance, validated_data)

SupportTeamMemberTicketSerializer : type[BaseSupportTeamMemberTicketSerializer] = import_string(get_settings().get('SupportTeamMemberTicketSerializer','dj_tickets.rest.serializers.BaseSupportTeamMemberTicketSerializer'))


class BaseSupportTeamAdminTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',

            'observers',
            'support_team',
            'status',
            'support_team_member',

            'content',
            'creator',
            'created',
            'modified',
        ]
        extra_kwargs = {
            'content': { 'read_only':True },
            'creator': { 'read_only':True },
        }

    def to_representation(self, instance:AbstractTicket):
        data = super().to_representation(instance)

        data['observers'] = UserDetailSerializer(instance.observers,many=True).data
        data['creator'] = UserDetailSerializer(instance.creator).data

        return data

SupportTeamAdminTicketSerializer : type[BaseSupportTeamAdminTicketSerializer] = import_string(get_settings().get('SupportTeamAdminTicketSerializer','dj_tickets.rest.serializers.BaseSupportTeamAdminTicketSerializer'))


class BaseChangeStatusTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',
            'status',
        ]

ChangeStatusTicketSerializer : type[BaseChangeStatusTicketSerializer] = import_string(get_settings().get('ChangeStatusTicketSerializer','dj_tickets.rest.serializers.BaseChangeStatusTicketSerializer'))


class BaseAssignTeamTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',
            'support_team',
        ]

AssignTeamTicketSerializer : type[BaseAssignTeamTicketSerializer] = import_string(get_settings().get('AssignTeamTicketSerializer','dj_tickets.rest.serializers.BaseAssignTeamTicketSerializer'))




class BaseAssignTeamMemberTicketSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Ticket
        fields = [
            'id',
            'support_team_member',
        ]
    
    def validate_support_team_member(self, value):
        if self.instance.support_team:
            if value and value.support_team != self.instance.support_team:
                raise serializers.ValidationError('Cannot assign ticket to member not exists in its support team.')
            self.instance.status = Ticket.Status.IN_PROGRESS
            return value
        
        raise serializers.ValidationError('Assign a support team firstly than assign a member.')

AssignTeamMemberTicketSerializer : type[BaseAssignTeamMemberTicketSerializer] = import_string(get_settings().get('AssignTeamMemberTicketSerializer','dj_tickets.rest.serializers.BaseAssignTeamMemberTicketSerializer'))



class BaseSupportTeamSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = SupportTeam
        fields = [
            'id',
            'name',
            'created',
            'modified',
        ]

SupportTeamSerializer : type[BaseSupportTeamSerializer] = import_string(get_settings().get('SupportTeamSerializer','dj_tickets.rest.serializers.BaseSupportTeamSerializer'))


class BaseSupportTeamMemberSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = SupportTeamMember
        fields = [
            'id',
            'user',
            'support_team',
            'is_admin',
            'created',
            'modified',
        ]
    
    def to_representation(self, instance):
        data = super().to_representation(instance)
        data['user']  = UserDetailSerializer(instance.user).data
        data['support_team'] = SupportTeamSerializer(instance.support_team).data
        return data
    
    def update(self, instance, validated_data):
        validated_data.pop('is_admin',None)
        validated_data.pop('user',None)
        return super().update(instance, validated_data)

SupportTeamMemberSerializer : type[BaseSupportTeamMemberSerializer] = import_string(get_settings().get('SupportTeamMemberSerializer','dj_tickets.rest.serializers.BaseSupportTeamMemberSerializer'))


class BaseTicketMessageSerializer(serializers.ModelSerializer):

    user = UserDetailSerializer(read_only=True)
    
    class Meta:
        model = TicketMessage
        fields = [
            'id',
            'content',

            'ticket',
            'user',
            'is_creator',
            'is_readed',
            'created',
            'modified',
        ]

        extra_kwargs = {
            'ticket': { 'read_only':True },
            'is_readed': { 'read_only':True },
            'user': { 'read_only':True },
            'is_creator': { 'read_only':True },
        }

    def update(self, instance, validated_data):
        validated_data.pop('user',None)
        return super().update(instance, validated_data)
    
    def create(self, validated_data):
        validated_data['user'] = self.context['request'].user
        return super().create(validated_data)

TicketMessageSerializer : type[BaseTicketMessageSerializer] = import_string(get_settings().get('TicketMessageSerializer','dj_tickets.rest.serializers.BaseTicketMessageSerializer'))

