from django.shortcuts import render, get_object_or_404
from django.db.models import Q, Min, Max

from rest_framework import (
    viewsets,
    mixins,
    permissions, 
    decorators,
    request,
    response,
    serializers,
    exceptions
)
from django_filters.rest_framework import DjangoFilterBackend

from .serializers import (
    SupportTeamMemberTicketSerializer,
    SupportTeamMemberSerializer,
    SupportTeamSerializer,
    SupportTeamAdminTicketSerializer,
    AssignTeamTicketSerializer,
    AssignTeamMemberTicketSerializer,
    ChangeStatusTicketSerializer,
    TicketMessageSerializer,
    ClientTicketSerializer,
)
from dj_tickets.core.models import (
    Ticket,
    SupportTeam,
    SupportTeamMember,
    TicketMessage,
)
from .permissions import (
    IsSupportAdmin, 
    IsSupportTeamMember,
    CanAccessToTicketMessages,
    CanChatTicketMessages,
    CanDeleteTicket,
    CanAccessToSupportTeamMember,
)
from dj_tickets.core.utils import CustomPageNumberPagination


class ClientTicketViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.CreateModelMixin,
    viewsets.GenericViewSet
):

    permission_classes=[permissions.IsAuthenticated]
    pagination_class=CustomPageNumberPagination

    queryset = Ticket.objects.all()
    serializer_class = ClientTicketSerializer

    def get_queryset(self):
        return super().get_queryset().filter(creator = self.request.user)


class SupportTeamMemberTicketViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    viewsets.GenericViewSet
):

    permission_classes=[permissions.IsAuthenticated,IsSupportTeamMember]
    pagination_class=CustomPageNumberPagination

    queryset = Ticket.objects.all()
    serializer_class = SupportTeamMemberTicketSerializer

    def get_queryset(self):
        return super().get_queryset().filter(support_team_member__user = self.request.user)


class SupportTeamAdminTicketViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    viewsets.GenericViewSet
):

    permission_classes=[permissions.IsAuthenticated,IsSupportTeamMember]
    pagination_class=CustomPageNumberPagination

    queryset = Ticket.objects.all()
    serializer_class = SupportTeamAdminTicketSerializer


class SupportTeamMemberViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    viewsets.GenericViewSet
):

    permission_classes=[permissions.IsAuthenticated, CanAccessToSupportTeamMember]

    queryset = SupportTeamMember.objects.all()
    serializer_class = SupportTeamMemberSerializer
    pagination_class=CustomPageNumberPagination


class SupportTeamViewSet(viewsets.ModelViewSet):

    permission_classes=[permissions.IsAuthenticated, IsSupportAdmin]

    queryset = SupportTeam.objects.all()
    serializer_class = SupportTeamSerializer
    pagination_class=CustomPageNumberPagination


class TicketMessageViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    viewsets.GenericViewSet
):
    permission_classes=[permissions.IsAuthenticated,CanAccessToTicketMessages,CanChatTicketMessages]

    queryset = TicketMessage.objects.all()
    serializer_class = TicketMessageSerializer
    pagination_class=CustomPageNumberPagination

    def initialize_request(self, request, *args, **kwargs):
        if hasattr(request.user,'support_team_member'):
            if request.user.support_team_member.is_admin:
                self.ticket = get_object_or_404(Ticket,pk=self.kwargs['ticket_id'])
            else:
                self.ticket = get_object_or_404(Ticket,Q(
                    Q(pk=self.kwargs['ticket_id']) &
                    Q(
                        Q(support_team_member=request.user.support_team_member) |
                        Q(creator=request.user)
                    )
                ))
        else:
            self.ticket = get_object_or_404(Ticket,Q(
                Q(pk=self.kwargs['ticket_id']) &
                Q(creator=request.user)
            ))
        return super().initialize_request(request, *args, **kwargs)

    def get_queryset(self):
        return super().get_queryset().filter(ticket=self.ticket)

    @decorators.action(
        detail=True,
        methods=['GET'],
    )
    def mark_as_readed(self,request,ticket_id,pk):
        instance = self.get_object()
        # make all before messages as readed
        if request.user == self.ticket.creator:
            self.get_queryset().exclude(
                user=self.ticket.creator,created__lte=instance.created
            ).update(is_readed=True)
        else:
            self.get_queryset().filter(
                user=self.ticket.creator,created__lte=instance.created
            ).update(is_readed=True)

        return self.retrieve(request,pk)
    
    @decorators.action(
        detail=False,
        methods=['GET'],
    )
    def is_there_unreads(self,request,ticket_id):
        is_there_unreads = False
        if request.user == self.ticket.creator:
            is_there_unreads = self.get_queryset().exclude(
                user=self.ticket.creator
            ).exists()
        else:
            is_there_unreads = self.get_queryset().filter(
                user=self.ticket.creator
            ).exists()
        return response.Response({'is_there_unreads':is_there_unreads})
    
    def perform_create(self, serializer):
        serializer.save(ticket=self.ticket,user=self.request.user)

